﻿namespace MVCDemo
{
    using System.Web.Mvc;
    using System.Web.Routing;
    using MVCDemo.Models.Configuration;
    using MVCDemo.Models.Configuration.Rules;
    using MVCDemo.Models.Configuration.Sources;
    using MVCDemo.Models.MetadataProviders;

    // Note: For instructions on enabling IIS6 or IIS7 classic mode, 
    // visit http://go.microsoft.com/?LinkId=9394801

    public class MvcApplication : System.Web.HttpApplication
    {
        public static void RegisterGlobalFilters(GlobalFilterCollection filters)
        {
            filters.Add(new HandleErrorAttribute());
        }

        public static void RegisterRoutes(RouteCollection routes)
        {
            routes.IgnoreRoute("{resource}.axd/{*pathInfo}");

            routes.MapRoute(
                "User", // Route name
                "{controller}/{action}/{id}", // URL with parameters
                new {
                    controller = "UserProfiles",
                    action = "Index",
                    id = UrlParameter.Optional
                } // Parameter defaults
            );
        }

        protected void Application_Start()
        {
            AreaRegistration.RegisterAllAreas();

            RegisterGlobalFilters(GlobalFilters.Filters);
            RegisterRoutes(RouteTable.Routes);

            var modelProvider = new ModelConfigurationProvider();

            modelProvider.Sources.Add(
                new XmlModelConfigurationSource(
                    Server.MapPath("~/App_Data/ModelMetadata/")));

            var manager = new ModelMetadataProvidersManager(ModelMetadataProviders.Current);
            manager.Providers.Add(modelProvider.MetadataProvider);
            ModelMetadataProviders.Current = manager;

            modelProvider.AddValidatiorRule("BlockHtml", new BlockHtmlRule());
            modelProvider.AddValidatiorRule("Equal", new EqualRule());
            modelProvider.AddValidatiorRule("StringLengthRange", new StringLengthRangeRule());

            ModelValidatorProviders.Providers.Insert(0, modelProvider.ValidatorProvider);
        }
    }
}