﻿/// Copyright (c) 2011 Andrey Veselov. All rights reserved.
/// WebSite: http://andrey.moveax.ru 
/// Email: andrey@moveax.ru
/// This source is subject to the Microsoft Public License (Ms-PL).

namespace MVCDemo.Models.Configuration
{
    using System;
    using System.Collections.Generic;
    using System.Web.Mvc;
    using System.Xml.Linq;
    using MVCDemo.Models.Configuration.Rules;
    using MVCDemo.Models.Configuration.Sources;
    using MVCDemo.Models.MetadataProviders;

    public partial class ModelConfigurationProvider
    {
        #region Private fields

        private readonly Dictionary<Type, ModelConfiguraion> _cache;

        private readonly ModelValidatorProviderAdapter _validatorProvider;

        #endregion

        public ModelConfigurationProvider()
        {
            this.Sources = new List<IModelConfigurationSource>();
            this._cache = new Dictionary<Type, ModelConfiguraion>();

            this.MetadataProvider = new ModelMetadataProviderAdapter(this);
            this._validatorProvider = new ModelValidatorProviderAdapter(this);
        }

        #region Properties

        public IModelMetadataProvider MetadataProvider { get; private set; }

        public ModelValidatorProvider ValidatorProvider
        {
            get { return this._validatorProvider; }
        }

        public List<IModelConfigurationSource> Sources { get; private set; }

        #endregion

        public void AddValidatiorRule(string ruleName, IModelValidatorRule rule)
        {
            this._validatorProvider.Rules.Add(ruleName, rule);
        }

        private ModelConfiguraion GetModelConfiguraion(Type modelType)
        {
            foreach (var modelConfiguraion in this._cache) {
                if (modelConfiguraion.Key.Equals(modelType)) {
                    return modelConfiguraion.Value;
                }
            }

            foreach (var source in this.Sources) {
                bool useCache = false;
                XElement config = source.GetModelConfiguration(modelType, out useCache);
                if (config != null) {
                    var data = new ModelConfiguraion(config);

                    if (useCache) {
                        this._cache.Add(modelType, data);
                    }

                    return data;
                }
            }

            return null;
        }

        private class ModelConfiguraion
        {
            public ModelConfiguraion(XElement source)
            {
                this.Source = source;

                var resourceAttr = this.Source.Attribute("resource");
                if (resourceAttr != null) {
                    this.DefaultResourceType = Type.GetType(resourceAttr.Value, throwOnError: true);
                }
            }

            public XElement Source { get; private set; }

            public Type DefaultResourceType { get; private set; }
        }
    }
}