﻿/// Copyright (c) 2011 Andrey Veselov. All rights reserved.
/// WebSite: http://andrey.moveax.ru 
/// Email: andrey@moveax.ru
/// This source is subject to the Microsoft Public License (Ms-PL).

namespace MVCDemo.Models.Configuration
{
    using System;
    using System.Collections.Generic;
    using System.IO;
    using System.Linq;
    using System.Web.Mvc;
    using MVCDemo.Models.Configuration.Rules;
    using MVCDemo.Models.Extensions;

    public partial class ModelConfigurationProvider
    {
        private class ModelValidatorProviderAdapter : ModelValidatorProvider
        {
            private readonly ModelConfigurationProvider _cfgProvider;

            public ModelValidatorProviderAdapter(ModelConfigurationProvider cfgProvider)
            {
                if (cfgProvider == null) {
                    throw new ArgumentNullException();
                }

                this._cfgProvider = cfgProvider;

                this.Rules = new Dictionary<string, IModelValidatorRule>();

                // add default rules
                this.Rules.Add("Require", new RequireRule());
                this.Rules.Add("Range", new RangeRule());
                this.Rules.Add("RegExp", new RegularExpressionRule());
                this.Rules.Add("StringLength", new StringLengthRule());
                this.Rules.Add("Remote", new RemoteRule());
                this.Rules.Add("CustomValidation", new CustomValidationRule());
            }

            public Dictionary<string, IModelValidatorRule> Rules { get; private set; }

            #region ModelValidatorProvider method override

            public override IEnumerable<ModelValidator> GetValidators(
                ModelMetadata metadata, ControllerContext context)
            {
                var cfgData = this._cfgProvider.GetModelConfiguraion(metadata.ContainerType);
                if (cfgData == null) {
                    yield break;
                }

                var xmlPropertyElement = cfgData.Source.Elements("property").FirstOrDefault(
                    element => string.Equals(element.Attribute("name").Value, metadata.PropertyName));

                if (xmlPropertyElement == null) {
                    yield break;
                }

                // Get rules for the property using XML attributes.
                foreach (var element in xmlPropertyElement.Elements()) {
                    if (!string.Equals(element.Name.LocalName, "rule")) {
                        continue;
                    }

                    string ruleName = element.GetValueOrNull("name");
                    if (ruleName == null) {
                        throw new InvalidDataException(
                            string.Format("Incorrect rule '{0}'.", element));
                    }

                    IModelValidatorRule factoryMethod = this.Rules[ruleName];
                    yield return factoryMethod.Create(
                        element, cfgData.DefaultResourceType, metadata, context);
                }
            }

            #endregion
        }
    }
}