﻿/// Copyright (c) 2011 Andrey Veselov. All rights reserved.
/// WebSite: http://andrey.moveax.ru 
/// Email: andrey@moveax.ru
/// This source is subject to the Microsoft Public License (Ms-PL).

namespace MVCDemo.Models.Configuration.Rules
{
    using System;
    using System.ComponentModel.DataAnnotations;
    using System.IO;
    using System.Web.Mvc;
    using System.Xml.Linq;
    using MVCDemo.Models.Extensions;

    public class CustomValidationRule : ModelValidatorRule
    {
        public override ModelValidator Create(
            XElement xmlElement, Type defaultResourceType, ModelMetadata metadata, ControllerContext context)
        {
            string classTypeName = xmlElement.GetValueOrNull("type");
            if (classTypeName == null) {
                throw new InvalidDataException(
                    string.Format("The class type was not set. Element: {0}", xmlElement));
            }

            Type classType = Type.GetType(classTypeName, throwOnError: true);
            if (classType == null) {
                throw new InvalidDataException(
                    string.Format("Unknown class type. Element: {0}", xmlElement));
            }

            string method = xmlElement.GetValueOrNull("method");
            if (method == null) {
                throw new InvalidDataException(
                    string.Format("The action was not set. Element: {0}", xmlElement));
            }

            var attribute = new CustomValidationAttribute(classType, method);
            this.BindErrorMessageToAttribte(attribute, xmlElement, defaultResourceType);

            return new DataAnnotationsModelValidator<CustomValidationAttribute>(metadata, context, attribute);
        }
    }
}