﻿/// Copyright (c) 2011 Andrey Veselov. All rights reserved.
/// WebSite: http://andrey.moveax.ru 
/// Email: andrey@moveax.ru
/// This source is subject to the Microsoft Public License (Ms-PL).

namespace MVCDemo.Models.Configuration.Rules
{
    using System;
    using System.ComponentModel.DataAnnotations;
    using System.IO;
    using System.Web.Mvc;
    using System.Xml.Linq;
    using MVCDemo.Models.Extensions;

    public class RangeRule : ModelValidatorRule
    {
        public override ModelValidator Create(
            XElement xmlElement, Type defaultResourceType, ModelMetadata metadata, ControllerContext context)
        {
            string minValue = xmlElement.GetValueOrNull("min");
            if (minValue == null) {
                throw new InvalidDataException(
                    string.Format("Min value can't be null. Element: {0}.", xmlElement));
            }

            string maxValue = xmlElement.GetValueOrNull("max");
            if (maxValue == null) {
                throw new InvalidDataException(
                    string.Format("Max value can't be null. Element: {0}.", xmlElement));
            }

            string typeName = xmlElement.GetValueOrNull("type");
            if (typeName == null) {
                typeName = "System.Int32";
            }

            Type rangeType = Type.GetType(typeName);
            if (rangeType == null) {
                throw new InvalidDataException(
                    string.Format("Unknown type: {0}. Element: {1}.", typeName, xmlElement));
            }

            var attribute = new RangeAttribute(rangeType, minValue, maxValue);
            this.BindErrorMessageToAttribte(attribute, xmlElement, defaultResourceType);

            return new RangeAttributeAdapter(metadata, context, attribute);
        }
    }
}