﻿/// Copyright (c) 2011 Andrey Veselov. All rights reserved.
/// WebSite: http://andrey.moveax.ru 
/// Email: andrey@moveax.ru
/// This source is subject to the Microsoft Public License (Ms-PL).

namespace MVCDemo.Models.Configuration.Rules
{
    using System;
    using System.IO;
    using System.Web.Mvc;
    using System.Xml.Linq;
    using MVCDemo.Models.Extensions;

    public class RemoteRule : ModelValidatorRule
    {
        public override ModelValidator Create(
            XElement xmlElement, Type defaultResourceType, ModelMetadata metadata, ControllerContext context)
        {
            string controller = xmlElement.GetValueOrNull("controller");
            if (controller == null) {
                throw new InvalidDataException(
                    string.Format("The controller was not set. Element: {0}", xmlElement));
            }

            string action = xmlElement.GetValueOrNull("action");
            if (action == null) {
                throw new InvalidDataException(
                    string.Format("The action was not set. Element: {0}", xmlElement));
            }
            string httpMethod = xmlElement.GetValueOrNull("method"); 

            var attribute = new RemoteAttribute(action, controller);
            this.BindErrorMessageToAttribte(attribute, xmlElement, defaultResourceType);
            if (httpMethod != null) {
                attribute.HttpMethod = httpMethod;
            }

            return new DataAnnotationsModelValidator<RemoteAttribute>(metadata, context, attribute);
        }
    }
}