﻿/// Copyright (c) 2011 Andrey Veselov. All rights reserved.
/// WebSite: http://andrey.moveax.ru 
/// Email: andrey@moveax.ru
/// This source is subject to the Microsoft Public License (Ms-PL).

namespace MVCDemo.Models.Configuration.Rules
{
    using System;
    using System.IO;
    using System.Web.Mvc;
    using System.Xml.Linq;
    using MVCDemo.Attributes.Validation;
    using MVCDemo.Models.Extensions;

    public class StringLengthRangeRule : ModelValidatorRule
    {
        public override ModelValidator Create(
            XElement xmlElement, Type defaultResourceType, ModelMetadata metadata, ControllerContext context)
        {
            string min = xmlElement.GetValueOrNull("min");
            if (min == null) {
                throw new InvalidDataException(
                    string.Format("The min value was not set. Element: {0}", xmlElement));
            }

            string max = xmlElement.GetValueOrNull("max");
            if (min == null) {
                throw new InvalidDataException(
                    string.Format("The max value was not set. Element: {0}", xmlElement));
            }


            int minValue = Convert.ToInt32(min);
            int maxValue = Convert.ToInt32(max);

            var attribute = new StringLengthRangeAttribute(minValue, maxValue);
            this.BindErrorMessageToAttribte(attribute, xmlElement, defaultResourceType);

            return new DataAnnotationsModelValidator<StringLengthRangeAttribute>(metadata, context, attribute);
        }
    }
}