﻿/// Copyright (c) 2011 Andrey Veselov. All rights reserved.
/// WebSite: http://andrey.moveax.ru 
/// Email: andrey@moveax.ru
/// This source is subject to the Microsoft Public License (Ms-PL).

namespace MVCDemo.Models.Configuration.Sources
{
    using System;
    using System.IO;
    using System.Xml.Linq;

    public class XmlModelConfigurationSource : IModelConfigurationSource
    {
        private readonly string _xmlStoragePath;

        public XmlModelConfigurationSource(string xmlStoragePath)
        {
            if (!Directory.Exists(xmlStoragePath)) {
                throw new ArgumentException(
                    string.Format("Invalid directory {0}", xmlStoragePath));
            }

            this._xmlStoragePath =
                xmlStoragePath.EndsWith("\\") ?
                xmlStoragePath : string.Format("{0}\\", xmlStoragePath);
        }

        #region IModelConfigurationProvider Members

        public XElement GetModelConfiguration(Type modelType, out bool useCache)
        {
            if (modelType == null) {
                useCache = false;
                return null;
            }

            string metadataFile = string.Format(
                "{0}{1}.config", this._xmlStoragePath, modelType.FullName);

            if (!File.Exists(metadataFile)) {
                useCache = false;
                return null;
            }

            useCache = true;
            return XElement.Load(metadataFile);
        }

        #endregion
    }
}