﻿/// Copyright (c) 2011 Andrey Veselov. All rights reserved.
/// WebSite: http://andrey.moveax.ru 
/// Email: andrey@moveax.ru
/// This source is subject to the Microsoft Public License (Ms-PL).

namespace MVCDemo.Models.MetadataProviders
{
    using System;
    using System.Collections.Generic;
    using System.Web.Mvc;
 
    public class ModelMetadataProvidersManager : ModelMetadataProvider
    {
        private readonly ModelMetadataProvider _defaultProvider;

        public List<IModelMetadataProvider> Providers { get; private set; }

        public ModelMetadataProvidersManager(ModelMetadataProvider defaultProvider)
        {
            this.Providers = new List<IModelMetadataProvider>();

            this._defaultProvider =
                (defaultProvider != null) ?
                defaultProvider : new EmptyModelMetadataProvider();
        }

        #region ModelMetadataProvider methods

        public override IEnumerable<ModelMetadata> GetMetadataForProperties(
            object container, Type containerType)
        {
            foreach (var metadataProvider in this.Providers) {
                if (metadataProvider.IsAbleToProvideMetadata(containerType)) {
                    return metadataProvider.GetMetadataForProperties(container, containerType);
                }
            }

            return this._defaultProvider.GetMetadataForProperties(container, containerType);
        }

        public override ModelMetadata GetMetadataForProperty(
            Func<object> modelAccessor, Type containerType, string propertyName)
        {
            foreach (var metadataProvider in this.Providers) {
                if (metadataProvider.IsAbleToProvideMetadata(containerType)) {
                    return metadataProvider.GetMetadataForProperty(modelAccessor, containerType, propertyName);
                }
            }

            return this._defaultProvider.GetMetadataForProperty(modelAccessor, containerType, propertyName);
        }

        public override ModelMetadata GetMetadataForType(
            Func<object> modelAccessor, Type modelType)
        {
            foreach (var metadataProvider in this.Providers) {
                if (metadataProvider.IsAbleToProvideMetadata(modelType)) {
                    return metadataProvider.GetMetadataForType(modelAccessor, modelType);
                }
            }

            return this._defaultProvider.GetMetadataForType(modelAccessor, modelType);
        }

        #endregion
    }
}