﻿namespace MVCDemo.Models
{
    using System;
    using System.Collections.Generic;

    public class PaymentRepository
    {
        private static List<PaymentModel> _payments = new List<PaymentModel>();

        static PaymentRepository()
        {
            var userProfiles = new UserRepository();

            _payments.Add(new PaymentModel() {
                Id = Guid.NewGuid(),
                UserLogin = userProfiles.Profiles[0].Login,
                Date = DateTime.Now,
                Amount = 1000.0f,
                Source = PaymentModel.PaymentSource.Direct,
                Info = string.Empty
            });
        }

        public List<PaymentModel> Payments
        {
            get { return _payments; }
        }

        public void Add(
            PaymentModel newPayment,
            PaymentModel.PaymentSource paymentSource = PaymentModel.PaymentSource.Unknown)
        {
            if (newPayment == null) {
                throw new ArgumentNullException();
            }

            // Validate the payment source value
            if ((paymentSource == PaymentModel.PaymentSource.Unknown) &&
                (newPayment.Source == PaymentModel.PaymentSource.Unknown)) {
                    // TODO: Replace the exception with the logging code
                    throw new ArgumentException("Unknown payment source.");
            }

            // Validate the user login
            var userProfiles = new UserRepository();
            if (!userProfiles.IsLoginExists(newPayment.UserLogin)) {
                // TODO: Replace the exception with the logging code
                throw new ArgumentException(
                    string.Format("Invalid login: {0}", newPayment.UserLogin));
            }

            // Update and store the payment information
            newPayment.Id = Guid.NewGuid();
            if (paymentSource != PaymentModel.PaymentSource.Unknown) {
                newPayment.Source = paymentSource;
            }

            _payments.Add(newPayment);
        }
    }
}