﻿namespace MVCDemo.Models
{
    using System.ComponentModel.DataAnnotations;
    using System.Web.Mvc;
    using MVCDemo.Attributes.Validation;
    using MVCDemo.Resources.Models;
    using MVCDemo.Resources.Shared;

    public class UserProfileModel
    {
        [Display(Name = "Login", ResourceType = typeof(UserProfileRes))]
        [DataType(DataType.Text)]
        [Required(ErrorMessageResourceName = "FieldIsRequired",
            ErrorMessageResourceType = typeof(ErrorsRes))]
        [StringLengthRange(3, 64,
            ErrorMessageResourceName = "InvalidStringLength",
            ErrorMessageResourceType = typeof(ErrorsRes))]
        [BlockHtml(ErrorMessageResourceName = "HtmlTagsNotAllowed",
            ErrorMessageResourceType = typeof(ErrorsRes))]
        [AllowHtml]
        public virtual string Login { get; set; }

        [Display(Name = "DisplayName", ResourceType = typeof(UserProfileRes))]
        [DataType(DataType.Text)]
        [Required(ErrorMessageResourceName = "FieldIsRequired",
            ErrorMessageResourceType = typeof(ErrorsRes))]
        [StringLengthRange(3, 32,
            ErrorMessageResourceName = "InvalidStringLength",
            ErrorMessageResourceType = typeof(ErrorsRes))]
        [BlockHtml(ErrorMessageResourceName = "HtmlTagsNotAllowed",
            ErrorMessageResourceType = typeof(ErrorsRes))]
        public string DisplayName { get; set; }

        [Display(Name = "Email", ResourceType = typeof(UserProfileRes))]
        [DataType(DataType.EmailAddress)]
        [Required(ErrorMessageResourceName = "FieldIsRequired",
            ErrorMessageResourceType = typeof(ErrorsRes))]
        [RegularExpression(@"[\w\.-]*[a-zA-Z0-9_]@[\w\.-]*[a-zA-Z0-9]\.[a-zA-Z][a-zA-Z\.]*[a-zA-Z]",
            ErrorMessageResourceName = "InvalidEmailAddress",
            ErrorMessageResourceType = typeof(ErrorsRes))]
        [StringLength(64,
             ErrorMessageResourceName = "StringTooLong",
             ErrorMessageResourceType = typeof(ErrorsRes))]
        public string Email { get; set; }

        [Display(Name = "Homepage", ResourceType = typeof(UserProfileRes))]
        [DataType(DataType.Url)]
        [RegularExpression(@"(http(s)?://)?([\w-]+\.)+[\w-]+(/[\w- ;,./?%&=]*)?",
            ErrorMessageResourceName = "InvalidUrl",
            ErrorMessageResourceType = typeof(ErrorsRes))]
        [StringLength(96,
            ErrorMessageResourceName = "StringTooLong",
            ErrorMessageResourceType = typeof(ErrorsRes))]
        public string Homepage { get; set; }

        [Display(Name = "Description", ResourceType = typeof(UserProfileRes))]
        [DataType(DataType.MultilineText)]
        [StringLength(255,
            ErrorMessageResourceName = "StringTooLong",
            ErrorMessageResourceType = typeof(ErrorsRes))]
        [AllowHtml]
        public string Description { get; set; }
    }
}